require 'git'

module BBFlow
  class Misc
    class << self
      extend Memoist

      # @param [String] url
      #
      # @return [void]
      def open_url(url)
        if Gem::Platform.local.os == 'darwin'
          execute("open #{url}")
        else
          # FIXME: I'm not sure what's the proper way to open URLs on either Linux
          #        or Windows. If it's important for you, don't hesitate to edit.
          puts(url)
        end
      end

      # @param [Array<Object>] args
      #
      # @return [String]
      def execute(*args)
        Open3.popen3(*args) { |_i, o| return o.read }
      end

      # @param [String] text
      #
      # @return [String]
      def edit(text = '')
        file = Tempfile.new(%w(bb-flow .md))
        file.write(text)
        file.flush

        system(*[*Misc.editor, file.path])

        File.read(file.path)
      ensure
        file.close
        file.unlink
      end

      # @return [Git::Base]
      memoize def git
        Git.open(Dir.pwd)
      rescue ArgumentError
        Printer.panic("This directory (#{__dir__}) is not a git repository.")
      end

      # @return [Array<String>] editor with arguments.
      memoize def editor
        [`git config --global core.editor`.chomp.split(/\s+/), ['vi']].reject(&:empty?).first
      end

      # @param [String] word
      # @param [Fixnum] count
      #
      # @return [String]
      def pluralize(word, count)
        count == 1 ? word : word + 's'
      end
    end
  end
end
