module BBFlow
  class Printer
    class << self
      # @param [String] question
      #
      # @return [String] answer
      def ask(question)
        puts
        note(question)
        print('> ')
        gets.strip
      end

      # @param [String] question
      #
      # @return [Boolean] positive answer
      def simple_question(question)
        yes = 'Yes, please'
        no = 'No, thank you'

        select_item([yes, no], question, formatter: :to_s.to_proc) == yes
      end

      # @param [String] message
      #
      # @return [void]
      def panic(message)
        error(message)
        abort
      end

      # @param [String] message
      #
      # @return [void]
      def error(message)
        puts("[ERROR] #{message}".colorize(:red))
      end

      # @param [String] message
      #
      # @return [void]
      def success(message)
        puts(message.colorize(:green))
      end

      # @param [String] message
      #
      # @return [void]
      def note(message)
        puts(message.colorize(:yellow))
      end

      # @param [String] message
      #
      # @return [void]
      def auxiliary(message)
        puts(message.colorize(:light_black))
      end

      # @param [Array<#name>] items
      # @param [String] title
      #
      # @return [#name]
      def select_item(items, title, formatter: :name.to_proc)
        puts
        note(title)
        auxiliary('Move: arrows, j/k, or Ctrl-N/P    Finish: return')
        Ansi::Selector.select(items, formatter: formatter)
      end

      # @param [Array<#name>] items
      # @param [String] title
      #
      # @return [Array<#name>]
      def select_items(items, title, formatter: :name.to_proc, preselected: [])
        puts
        note(title)
        # TODO: Remove after implementing it in ansi-select.
        auxiliary('Move: arrows or j/k    Toggle: space    Finish: return')
        Ansi::Selector.multi_select(items, formatter: formatter, preselected: preselected)
      end
    end
  end
end
