module BBFlow
  class Trello
    class << self
      extend BBFlow::Persistent
      extend Memoist

      # @return [Array<Trello::Card>]
      memoize def cards
        puts 'Fetching Trello cards...'
        lists.flat_map { |list| list.cards.target }.select { |card| card.member_ids.include?(member.id) }
      end

      # @return [Array<String>]
      memoize def labels
        cards.flat_map { |card| card.labels.target }.map(&:name).map(&:downcase)
      end

      # @return [Trello::List]
      memoize def done_list
        trello::List.find(done_list_id)
      end

      # @param [Trello::Card] card
      #
      # @return [Array<String>]
      def format_card(card)
        prefix, _, title = card.name.rpartition(/(?<=\])\s*/) # At last closing square bracket.
        _score, _, tags = prefix.rpartition(/(?<=\))\s*/) # After last parentheses.

        [tags, title, card.short_url]
      end

      private

      # @return [Trello::Board]
      memoize def board
        trello::Board.find(board_id)
      end

      # @return [Array<Trello::List>]
      memoize def lists
        list_ids.map { |list| trello::List.find(list) }
      end

      # @return [Trello::Member]
      memoize def member
        trello::Member.find('me')
      end

      # @return [Class]
      memoize def trello
        ::Trello.configure do |config|
          config.developer_public_key = key
          config.member_token = member_token
        end

        ::Trello
      end

      # @return [String]
      persistent global def key
        Printer.ask('Go to https://trello.com/app-key and paste Key: ')
      end

      # @return [String]
      persistent global def member_token
        authorization_url = "http://trello.com/1/authorize?key=#{key}&response_type=token&expiration=never&scope=read,write"
        Printer.ask("Go to #{authorization_url}, click 'Allow', and paste Token")
      end

      # @return [String]
      persistent local def board_id
        Printer.select_item(trello::Board.all, 'Which Trello board do you work with?', formatter: ->(board) { [board.name, board.url] }).id
      end

      # @return [Array<String>]
      persistent local def list_ids
        Printer.select_items(board.lists, 'Which lists contain not finished - „in progress“ - cards?').map(&:id)
      end

      # @return [String]
      persistent local def done_list_id
        Printer.select_item(board.lists, 'Where should I move finished cards?').id
      end
    end
  end
end
